@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 #
 # This script is used by the braker.pl pipeline.
 # Please be extremely careful when changing this script because the braker.pl
@@ -14,6 +14,7 @@
 # Last modified by Katharina J. Hoff on Feb 21st 2018
 
 use strict;
+use warnings;
 use Getopt::Long;
 
 my $usage .= "$0 -- convert GFF file and sequence fasta file to minimal genbank format\n";
@@ -175,6 +176,13 @@ sub insertExon {
     }
 }
 
+# escape characters according GFF3 Format restrictions for Column 1: "seqid"  
+# see http://gmod.org/wiki/GFF3
+sub escape_seqid {
+    my ($seqID) = @_;
+    $seqID =~ s/([^a-zA-Z0-9\.:\^\*\$@!\+_\?\-\|])/sprintf("%%%02X",ord($1))/ge;
+    return $seqID;
+}
 
 #
 # Now write the data
@@ -193,20 +201,37 @@ while(<FASTA>) {
     
     $length = length $seq;
     $annotation = $annos{$seqname};
-    my $shortseqname = $seqname;
-    $shortseqname =~ s/\s.*//;
-
-    if (!defined($annotation) && defined($annos{$shortseqname})){
-	$seqnameerrcount++;
-	if ($seqnameerrcount <= 10){
-	    print STDERR "Sequence $seqname has no annotation but $shortseqname has. ";
-	    print STDERR "Assuming that space truncates name.\n";
-	}
-	if ($seqnameerrcount == 10){
-	    print STDERR "Supressing this error message from now on.\n";
-	}
-	$seqname = $shortseqname;
-	$annotation = $annos{$shortseqname};
+    my $annotatedseqname = $seqname;
+    
+    if (!defined($annotation)) {
+        my $shortseqname = $seqname;
+        $shortseqname =~ s/\s.*//;
+        $annotatedseqname = $shortseqname;
+        $annotation = $annos{$annotatedseqname}; # try the short sequence name (split at first whitespace)
+        my $usedshortseqname = defined($annotation);
+        
+        if (!defined($annotation)) { # check if seq id in gff-file contains escaped characters
+            $annotatedseqname = escape_seqid($seqname);
+            $annotation = $annos{$annotatedseqname}; # try the escaped sequence name (as demanded by GFF3 format)
+            
+            if (!defined($annotation)) {
+                $annotatedseqname = escape_seqid($shortseqname);
+                $annotation = $annos{$annotatedseqname}; # try the escaped short sequence name
+                $usedshortseqname = defined($annotation);
+            }
+        }
+        
+        if ($usedshortseqname) {
+            $seqnameerrcount++;
+            if ($seqnameerrcount <= 10) {
+                print STDERR "Sequence $seqname has no annotation but $shortseqname has. ";
+                print STDERR "Assuming that space truncates name.\n";
+                if ($seqnameerrcount == 10) {
+                    print STDERR "Supressing this error message from now on.\n";
+                }
+            }
+            $seqname = $shortseqname;
+        }
     }
     # For each UTR annotation check whether there is a CDS annoation with the same name and matching boundaries.
     # If yes, make an mRNA from the UTR and discard the UTR annotation.
@@ -276,7 +301,7 @@ while(<FASTA>) {
 	}
     }
 
-    my @nr_sort_fkeyarray = nrsort(values %{$annos{$seqname}});
+    my @nr_sort_fkeyarray = nrsort(values %{$annos{$annotatedseqname}});
 
     # extract only the cds annotations
     my @cdsfkeys = ();
@@ -357,6 +382,7 @@ while(<FASTA>) {
 		    $shiftedfeatures[$j] -= $offset if (!$connected);
 		}
 		$newseq = substr($seq, $currentSeqBegin - 1, $currentSeqEnd - $currentSeqBegin + 1 );
+		$newlength = length $newseq;
 		&printdata(@shiftedfeatures);
 		&printseq($newseq, $newlength) if (!$connected);
 	    }
@@ -473,43 +499,48 @@ sub printdata {
     $genename = shift @gbfkey;
     $strand = shift @gbfkey;
     if ($strand eq "-") {
-	$join = "complement(";
+		$join = "complement(";
     } else {
-	$join = "";
+		$join = "";
     }
     if (@gbfkey>2) {
-	$join .= "join(";
+		$join .= "join(";
     }
     for ($i=0; $i<@gbfkey; $i+=2) {
-	if ($i != 0) {
-	    $join .= ",";
-	}
-	$join .= "$gbfkey[$i]..$gbfkey[$i+1]";
+		if ($i != 0) {
+	    	$join .= ",";
+		}
+		$join .= "$gbfkey[$i]..$gbfkey[$i+1]";
     }
     if (@gbfkey>2) {
-	$join .= ")";
+		$join .= ")";
     }
     if ($strand eq "-") {
-	$join .= ")";
+		$join .= ")";
     }
     if ($fkey eq "mRNA"){
-	$genename = $genename;
-	$genename =~ s/^mRNA//;
-	print OUTPUT "     mRNA            ";
+		$genename = $genename;
+		$genename =~ s/^mRNA//;
+		print OUTPUT "     mRNA            ";
     } else {
-	$genename = $genename;
-	$genename =~ s/^CDS//;
-	print OUTPUT "     CDS             ";
+		$genename = $genename;
+		$genename =~ s/^CDS//;
+		print OUTPUT "     CDS             ";
     }
-    while (length $join > 0) {
-	print OUTPUT substr $join,0,59;
-	$join = substr $join, 59; 
-	if (length $join > 0) {
-	    print OUTPUT "\n                     ";
-	}
+    while (length($join) > 0) {
+    	if(length($join) >= 60){
+			print OUTPUT substr $join,0,59; # 59 seems to be about line length? 60?
+			$join = substr $join, 59; 
+		}else{
+			print OUTPUT substr $join,0,length($join);
+			$join = ""
+		}
+		if (length $join > 0) {
+	    	print OUTPUT "\n                     ";
+		}
     }
     if (length $genename > 0){
-	print OUTPUT "\n                     /gene=\"$genename\"";
+		print OUTPUT "\n                     /gene=\"$genename\"";
     }
     print OUTPUT "\n";
 }
@@ -556,17 +587,15 @@ sub printseq {
     print OUTPUT "\nORIGIN\n";
     $i = 1;
     $pos = 0;
-    while ($pos <= length $seq) {
+    while ($pos < length $seq) {
         $zahlzeile = "";
         for ($j=0; $j < 9-length "$i"; $j=$j+1) {
             print OUTPUT " ";
         }
         print OUTPUT "$i";
-        for ($j=0; $j < 6; $j=$j+1) {
+        for ($j=0; $j < 6 && $pos < length $seq; $j=$j+1) {
              $ten = substr $seq, $pos, 10;
-             if (length $ten > 0) {
-                 print OUTPUT " $ten";
-             }
+             print OUTPUT " $ten";
              $pos = $pos + 10; #$seq = substr $seq, 10;
         }
         print OUTPUT "\n";
