@@ -150,8 +150,17 @@ void appendRow(AlignmentRow **r1, const AlignmentRow *r2, int aliLen1, string si
     if (*r1 == NULL && r2 != NULL && (sigstr == "" || sigstr == r2->getSignature())){
 	*r1 = new AlignmentRow();
 	(*r1)->seqID = r2->seqID;
-	(*r1)->strand = r2->strand;
+    (*r1)->strand = r2->strand;
+
+
+    #ifdef TESTING
+	(*r1)->seqIDarchive = r2->seqIDarchive;
+	(*r1)->chrLen = r2->chrLen;				
+    #endif
+
     }
+
+    
     if ((*r1) && r2 &&
 	((*r1)->seqID != r2->seqID || r2->chrStart() <= (*r1)->chrEnd())){
 	// incomplatible sequence IDs or second row does not come after first row,
@@ -164,6 +173,11 @@ void appendRow(AlignmentRow **r1, const AlignmentRow *r2, int aliLen1, string si
 	    (*r1)->seqID = r2->seqID;
 	    (*r1)->strand = r2->strand;
 	    (*r1)->cumFragLen = 0;
+
+        #ifdef TESTING
+    	(*r1)->seqIDarchive = r2->seqIDarchive;
+    	(*r1)->chrLen = r2->chrLen;				
+        #endif
 	}
     }
     if (r2 && (sigstr == "" || sigstr == r2->getSignature())) {
@@ -426,6 +440,12 @@ void capAliSize(list<Alignment*> &alis, int maxRange){
 			    newAli->rows[ss] = new AlignmentRow();
 			    newAli->rows[ss]->seqID = a->rows[ss]->seqID;
 			    newAli->rows[ss]->strand = a->rows[ss]->strand;
+
+                #ifdef TESTING
+			    newAli->rows[ss]->seqIDarchive = a->rows[ss]->seqIDarchive;
+			    newAli->rows[ss]->chrLen = a->rows[ss]->chrLen;				
+                #endif
+                
 			    vector<fragment>::iterator first = a->rows[ss]->frags.begin(), 
 				last = a->rows[ss]->frags.end(), it, itCutPos;
 
@@ -868,3 +888,83 @@ void Alignment::pack(){
 
 int MsaSignature::maxSigStrLen = 0;
 
+#ifdef TESTING
+// convert the alignment according to new intervals/FASTA (start/end are in bed format)
+void Alignment::convertAlignment(int r, int start, int end) {
+    if(rows[r] != NULL){
+        int newChrLen = end - start + 1;
+
+        for(int f=0;f<rows[r]->frags.size();++f){
+            if(rows[r]->strand == plusstrand){
+                // cout << "converting " << start << " " << end << " " << rows[r]->seqID << " " << rows[r]->frags[f].chrPos << " " << start << " " << rows[r]->frags[f].chrPos - start << endl;
+                rows[r]->frags[f].chrPos -= start;
+            }
+            else{
+                rows[r]->frags[f].chrPos = rows[r]->chrLen - rows[r]->frags[f].chrPos;  // onto the positive strand
+                rows[r]->frags[f].chrPos -= start;
+                rows[r]->frags[f].chrPos =  newChrLen - rows[r]->frags[f].chrPos;       // back onto the negative strand (remark the new length is used)
+            }
+        }
+
+        rows[r]->seqID = rows[r]->seqID + "_" +  to_string(start) + "_" + to_string(end+1); // the identifier is again in BED format (open interval)
+        rows[r]->chrLen = newChrLen;
+    }        
+}
+#endif
+
+// StringAlignment stuff
+
+ostream& operator<< (ostream& strm, const StringAlignment &msa){
+    for (size_t s=0; s < msa.k; s++)
+        if (msa.rows[s].length() > 0)
+            strm << s << "\t" << msa.rows[s] << endl;
+    return strm;
+}
+
+void StringAlignment::insert(std::list<MsaInsertion> &insList, int maxInsertLen){
+    insList.sort();
+    int oldinsertpos = -1;
+
+    for (MsaInsertion msains : insList){
+        if (rows[msains.s].empty())
+            continue; // no insertion into empty rows
+
+	size_t insertsize = msains.insert.length();
+	if (insertsize > maxInsertLen){
+	    // round down to a multiple of 3 for reading-frame consistency
+	    // cout << "long insert " << insertsize << endl;
+	    insertsize = (maxInsertLen / 3) * 3;
+	}
+        if (msains.insertpos != oldinsertpos){
+            // new insert colums are always started with the longest
+            // inserts to make sure there is enough space
+            // insert gap colums for whole aligment
+            for (size_t s=0; s < k; s++) {
+                if (!rows[s].empty()){
+                    rows[s].insert(msains.insertpos, string(insertsize, '-'));
+                }
+            }
+            oldinsertpos = msains.insertpos;
+        }
+        // replace gaps left justified with insert
+        rows[msains.s].replace(msains.insertpos, insertsize, msains.insert, 0, insertsize);
+    }
+}
+
+size_t StringAlignment::removeGapOnlyCols(){
+    size_t numRemovedCols = 0;
+
+    for (int j = len - 1; j >= 0; --j){
+	// TODO: remove consecutive ranges of all-gap cols at once for efficiency
+        if (isGapOnlyCol(j)){
+            // delete column j
+            numRemovedCols++;
+            for (size_t s=0; s < k; s++) {
+                if (!rows[s].empty()){
+                    rows[s].erase(j, 1);
+                }
+            }
+        }
+    }
+    return numRemovedCols;
+}
