@@ -3,7 +3,7 @@
  * license: Artistic License, see file LICENSE.TXT or 
  *          http://www.opensource.org/licenses/artistic-license.php
  *
- * authors: Mario Stanke, mario@gobics.de
+ * authors: Mario Stanke, mario.stanke@uni-greifswald.de
  *
  * date    |   author      |  changes
  * --------|---------------|------------------------------------------
@@ -146,7 +146,6 @@ int main( int argc, char* argv[] ){
                 throw ProjectError("No query file specified. Type \"augustus\" for help.");
             }
             string filename =  Properties::getProperty(INPUTFILE_KEY);
-
             GBProcessor gbank(filename);
             if (Gene::gff3)
                 cout << "##gff-version 3" << endl;
@@ -198,7 +197,7 @@ int main( int argc, char* argv[] ){
                     cerr << "# Unknown option for strand: " << strandstr << endl;
             } catch (...){} // take default strand
 	
-            if(mea_prediction)
+            if (mea_prediction)
                 cout <<"# Using MEA approach (Maximizing expected accuracy)."<<endl;
 
             if (gbank.fileType() == fasta) {
@@ -315,7 +314,7 @@ void evaluateOnTestSet(AnnoSequence *annoseq, NAMGene &namgene, FeatureCollectio
 	
 	bool singlestrand = false; // use not the shadow states
 	try {
-            singlestrand = (Properties::getIntProperty("singlestrand") == 1);
+            singlestrand = Properties::getBoolProperty("singlestrand");
 	} catch (...) {}
 
 	cout << "# Predicted genes for sequence number " << dnaproben <<  " on ";
@@ -326,8 +325,9 @@ void evaluateOnTestSet(AnnoSequence *annoseq, NAMGene &namgene, FeatureCollectio
 	else 
 	    cout << "both strands" << endl;
 	if (singlestrand)
-	    cout << "# Overlapping genes on opposite strand were allowed." << endl;	    
-	genes = namgene.doViterbiPiecewise(sfc, annoseq, strand); 
+	    cout << "# Overlapping genes on opposite strand were allowed." << endl;
+        unsigned int num_pieces;
+        genes = namgene.doViterbiPiecewise(sfc, annoseq, strand, num_pieces);
 	ende = clock();
 	total += (double) (ende-anfang) / CLOCKS_PER_SEC;
 	//cout << "time " << (double) (ende-anfang) / CLOCKS_PER_SEC << ", seqlen=" << annoseq->length << endl;
@@ -412,7 +412,7 @@ void predictOnInputSequences(AnnoSequence *seq, NAMGene &namgene, FeatureCollect
 	    sfc.prepare(curseq, Constant::printHints && !(Gene::gff3));
 	    bool singlestrand = false; // use not the shadow states
 	    try {
-		singlestrand = (Properties::getIntProperty("singlestrand") == 1);
+		singlestrand = Properties::getBoolProperty("singlestrand");
 	    } catch (...) {}
 	
 	    cout << "# Predicted genes for sequence number " << dnaproben <<  " on ";
@@ -424,24 +424,30 @@ void predictOnInputSequences(AnnoSequence *seq, NAMGene &namgene, FeatureCollect
 		cout << "both strands" << endl;
 	    if (singlestrand)
 		cout << "# Overlapping genes on opposite strand are allowed." << endl;
-
-	    genes = namgene.doViterbiPiecewise(sfc, curseq, strand);
+            unsigned int num_pieces;
+            genes = namgene.doViterbiPiecewise(sfc, curseq, strand, num_pieces);
 
 	    try {
 		if (Properties::getBoolProperty("emiprobs")){ // get emission probs (special request from Ingo Ebersberger)
-		    try {
-			Annotation *a = new Annotation(), *olda = curseq->anno;
-			a->genes = genes;
-			curseq->anno = a;
-			namgene.setPathAndProb(curseq, extrinsicFeatures); 
-			cout << "# joint probability of gene structure and sequence in " << 
-			    Properties::getProperty(SPECIES_KEY) << " model: " << curseq->anno->emiProb << endl;
-			curseq->anno = olda;
-		    } catch (ProjectError &e){
-			cerr << e.getMessage() << endl << "Error: Could not compute the emission probabilities (--emiprobs)" << endl;
-		    }
-		}
-	    } catch (...) {}
+                    try {
+                        if (num_pieces > 1){
+                            int maxDNAPieceSize = Properties::getIntProperty( "maxDNAPieceSize" );
+                            cerr << "Error: --emiprobs option works only for input sequences of length at most maxDNAPieceSize (currently set to " <<
+                                maxDNAPieceSize << "). Use --maxDNAPieceSize to readjust." << endl;
+                        } else {
+                            Annotation *a = new Annotation(), *olda = curseq->anno;
+                            a->genes = genes;
+                            curseq->anno = a;
+                            namgene.setPathAndProb(curseq, extrinsicFeatures);
+                            cout << "# joint probability of gene structure and sequence in " <<
+                                Properties::getProperty(SPECIES_KEY) << " model: " << curseq->anno->emiProb << endl;
+                            curseq->anno = olda;
+                        }
+                    } catch (ProjectError &e){
+                        cerr << e.getMessage() << endl << "Error: Could not compute the emission probabilities (--emiprobs)" << endl;
+                    }
+                }
+            } catch (...) {}
 
 	    Transcript::destroyGeneSequence(genes); // don't need them anymore after they are printed
 	    successfull++;
@@ -552,8 +558,9 @@ void checkExtrinsicAccuracy(AnnoSequence *annoseq, NAMGene &namgene, FeatureColl
  * in memory when we actually need only a small part.
  */
 void cutRelevantPiece(AnnoSequence *annoseq){
-    int predictionStart, predictionEnd;
-    int seqlen = annoseq->length;
+    long predictionStart, predictionEnd;
+    long seqlen = annoseq->length;
+
     try {
 	predictionStart = Properties::getIntProperty( "predictionStart" ) - 1;
     } catch (...) {
