@@ -602,6 +602,7 @@ void GBSplitter::determineFileType(){
 	csin.seekg(0, ios::beg);
 	string line;
 	bool haveWrongChar=false;
+
 	while (csin && !haveWrongChar) {
 	    getline(csin, line);
 	    if (line[0]!='>') {
@@ -617,7 +618,7 @@ void GBSplitter::determineFileType(){
 	}
 	if (!haveWrongChar)
 	    ftype = fasta;
-	
+
 	csin.clear();
 	csin.seekg(0, ios::beg);
     }
@@ -644,7 +645,7 @@ GBPositions* GBSplitter::nextData( ){
 
 AnnoSequence *GBSplitter::getNextFASTASequence( ){
     char *sequence = NULL, *name = NULL;
-    int length;
+    long length;
     readOneFastaSeq(sin, sequence, name, length);
     if (sequence == NULL || length == 0)
 	return NULL;
@@ -674,7 +675,7 @@ Boolean GBSplitter::gotoEnd( ){
 
 
 Boolean GBSplitter::findPositions( GBPositions& pos ){
-    int fposb, fpose;
+    long fposb, fpose;
     fposb = sin.tellg();
     if( !gotoEnd( ) )
         return false;
@@ -697,12 +698,12 @@ Boolean GBSplitter::findPositions( GBPositions& pos ){
     istringstream isstrm( pos.buffer );
     char buf[GBMAXLINELEN];
     while( isstrm ){
-        int curpos = isstrm.tellg();
+        long curpos = isstrm.tellg();
         isstrm >> ws;
         isstrm.getline( buf, GBMAXLINELEN-1 );
 	if ((!sin.eof() && (sin.rdstate() & ios_base::failbit)) || strlen(buf) >= GBMAXLINELEN-2){
 	    throw GBError(string("Could not read the following line in Genbank file.\n") + buf 
-			  + "\nMaximum line length is \n" + itoa(GBMAXLINELEN-2) + ".\n");
+			  + "\nPossible reasons: line too long or file too large. Maximum line length is \n" + itoa(GBMAXLINELEN-2) + ".\n");
 	}
         char *src;
         char *rna;
