@@ -73,13 +73,12 @@ ostream& operator<<(ostream&out, Feature& feature){
 }
 
 istream& operator>>( istream& in, Feature& feature ){
-    char buff[1024]; 
-    char copybuff[1024];
+    char buff[8192], copybuff[8192];
     char *temp;
     const char *spos;
     try {
-	in.getline(buff, 1024);
-	strncpy(copybuff, buff, 1014);
+	in.getline(buff, sizeof(buff));
+	strcpy(copybuff, buff);
 
 	if (strstr(buff, "\t")==NULL) {
 	    throw ProjectError("Line not tab separated.");
@@ -232,7 +231,7 @@ istream& operator>>( istream& in, Feature& feature ){
     return in;
 }
 
-int Feature::offset = 0;
+long Feature::offset = 0;
 
 FeatureType Feature::getFeatureType(string typestring){
     FeatureType type;
