@@ -18,9 +18,9 @@
 #include <fstream>
 #include <getopt.h>     /* for getopt_long; standard getopt is in unistd.h */
 #include <stdlib.h>     /* for exit() */
-#include <mysql++.h>
+#include <mysql++/mysql++.h>
 #include <exception>
-#include <ssqls.h>
+#include <mysql++/ssqls.h>
 
 //#include <boost/iostreams/filtering_stream.hpp>
 //#include <boost/iostreams/filter/gzip.hpp>
@@ -161,7 +161,8 @@ int main( int argc, char* argv[] ){
 	    cout << "Looks like " << filename << " is in fasta format." << endl;
 	    createTableGenomes();
 	    char *sequence = NULL, *name = NULL;
-	    int length = 0, seqCount = 0, chunkCount = 0;
+	    long length = 0;
+            int seqCount = 0, chunkCount = 0;
 	    unsigned int lenCount = 0;
 	    //readOneFastaSeq(zin, sequence, name, length);
 	    readOneFastaSeq(ifstrm, sequence, name, length);
@@ -196,7 +197,7 @@ int main( int argc, char* argv[] ){
 			insertHint(f,species);
 			hintCount[f.seqname]++;
 		    }
-		} catch (ProjectError e){}
+		} catch (ProjectError &e){}
 	    }
 	    if(!hintCount.empty()){
 		cout << "inserted" << endl; 
